import { TYPES } from "./item.js";

export class RdDHotbar {

  static async createItemMacro(item, slot, armeCompetence = undefined) {
    let command = `game.system.rdd.RdDHotbar.rollMacro("${item.name}", "${item.type}"` + ((armeCompetence) ? `, "${armeCompetence}");` : `);`);
    let macro = game.macros.contents.find(m => (m.name === item.name) && (m.command === command));
    if (!macro) {
      macro = await Macro.create({
        name: item.name,
        type: "script",
        img: item.img,
        command: command
      }, { displaySheet: false })
    }
    await game.user.assignHotbarMacro(macro, slot);
  }

  static async addToHotbar(item, slot) {
    switch (item?.type ?? "") {
      case TYPES.arme:
        {
          // Les armes peuvent avoir plusieurs usages
          if (item.system.competence != "") {
            await this.createItemMacro(item, slot, "competence")
            slot++
          }
          if (item.system.lancer != "") {
            await this.createItemMacro(item, slot, "lancer")
            slot++
          }
          if (item.system.tir != "") {
            await this.createItemMacro(item, slot, "lancer")
            slot++
          }
        }
        return
      case TYPES.competence:
      case TYPES.competencecreature:
      default:
        await this.createItemMacro(item, slot)
        return
    }
  }

  /**
   * Create a macro when dropping an entity on the hotbar
   * Item      - open roll dialog for item
   * Actor     - open actor sheet
   * Journal   - open journal sheet
   */
  static initDropbar() {

    Hooks.on("hotbarDrop", (bar, documentData, slot) => {

      // Create item macro if rollable item - weapon, spell, prayer, trait, or skill
      if (documentData.type == "Item") {
        const item = fromUuidSync(documentData.uuid) ?? this.actor.items.get(documentData.uuid)
        console.log("DROP", documentData, item)
        switch (item?.type ?? "")
        {
          case TYPES.arme:
          case TYPES.competence:
          case TYPES.competencecreature:
            this.addToHotbar(item, slot)
            return false
        }
      }

      return true
    })
  }

  /** Roll macro */
  static rollMacro(itemName, itemType, competenceName) {
    const speaker = ChatMessage.getSpeaker();
    let actor;
    if (speaker.token) actor = game.actors.tokens[speaker.token];
    if (!actor) actor = game.actors.get(speaker.actor);
    if (!actor) {
      return ui.notifications.warn(`Impossible de trouver le personnage concerné`);
    }
    let item = actor?.items.find(it => it.name === itemName && it.type == itemType) ?? undefined;
    if (!item) {
      return ui.notifications.warn(`Impossible de trouver l'objet de cette macro`);
    }

    // Trigger the item roll
    switch (item.type) {
      case TYPES.arme:
        return actor.rollArme(item, competenceName);
      case TYPES.competence:
      case TYPES.competencecreature:
        return actor.rollCompetence(itemName);
    }
  }

}
